@@ -15,12 +15,12 @@ BASE_DIR = Path()
 TEMP_DIR = Path()
 HTML_DIR = Path()
 DATABASE_DIR = Path()
-CHECKM_DIR = Path()
-GTDBTK_DIR = Path()
-GENOME_NAME_MAPPING_FILE = ''
+CHECKM_DIR:None | Path = None
+GTDBTK_DIR:None | Path = None
+GENOME_NAME_MAPPING_FILE:None | Path = None
 CONTIG_FILES = []
 GENOME_NAMES = []
-LOG_FILE = ''
+LOG_FILE:None | Path = None
 
 MULTI_MODE = False
 RENAME_CONTIGS = False
@@ -36,6 +36,7 @@ FORCE_ANNOTATOR = 2
 ANNOTATOR_STATUS = {'antismash': RUN_ANNOTATOR,
                      'aragorn': RUN_ANNOTATOR,
                      'cdd': RUN_ANNOTATOR,
+                     'checkm': RUN_ANNOTATOR,
                      'cmscan': RUN_ANNOTATOR,
                      'diamond_and_blastn': RUN_ANNOTATOR,
                      'hmm': RUN_ANNOTATOR,
@@ -116,6 +117,7 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
         log(f'Ready to create databases from scratch with tasks {DATABASE_TASKS}.')
         return
     else: # we're going to annotate genomes...
+        current_dir = Path(os.getcwd())
         # (1) collect contig files
         if ',' in contig_file:
             for f in contig_file.split(','):
@@ -127,6 +129,7 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
         else:
             contig_file = Path(contig_file).absolute()
             if contig_file.is_dir():
+                FILE_EXTENSION = file_extension
                 CONTIG_FILES = [f.absolute() for f in sorted(contig_file.glob(f'*{FILE_EXTENSION}')) if f.is_file()]
             else:
                 if contig_file.exists():
@@ -142,9 +145,17 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
             exit(1)
         # (2) manage file structure / folders
         TEMP_DIR = BASE_DIR / 'temp'
+        if checkm_dir:
+            if Path(checkm_dir).is_absolute():
+                CHECKM_DIR = Path(checkm_dir)
+            else:
+                CHECKM_DIR = current_dir / checkm_dir
+        if gtdbtk_dir:
+            if Path(gtdbtk_dir).is_absolute():
+                CHECKM_DIR = Path(gtdbtk_dir)
+            else:
+                GTDBTK_DIR = current_dir / gtdbtk_dir
         os.chdir(TEMP_DIR)
-        CHECKM_DIR = Path(checkm_dir).absolute()
-        GTDBTK_DIR = Path(gtdbtk_dir).absolute()
         GENOME_NAME_MAPPING_FILE = TEMP_DIR / 'genome.name.mapping.txt'
         HTML_DIR = BASE_DIR / 'html'
         for folder in (TEMP_DIR, HTML_DIR):
@@ -190,7 +201,6 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
                 exit()
         # (3) set some global variables
         MIN_CONTIG_LENGTH = int(min_contig_length)
-        FILE_EXTENSION = file_extension
         DELIMITER = delimiter
         PREFIX = prefix
         # (4) set up multitreading
@@ -264,7 +274,7 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
         log(f'Ready to annotate {len(CONTIG_FILES)} genomes in dir "{BASE_DIR}" with '
                   f'{CPUS_PER_GENOME} threads per genome.')
 
-def spawn_file(program_name, genome_id, base_dir = None) -> Path:
+def spawn_file(program_name, genome_id, base_dir = None, extension='') -> Path:
     """computes a Path genome_id.program_name or, if multimode==True, program_name/genome_id"""
     target_dir = Path(base_dir) if base_dir else TEMP_DIR
     if MULTI_MODE:
@@ -276,7 +286,7 @@ def spawn_file(program_name, genome_id, base_dir = None) -> Path:
                 dir.mkdir(exist_ok=True)
             else:
                 raise Exception("Use force to overwrite existing results")
-        return dir / genome_id
+        return dir / f'{genome_id}.{extension}' if extension else dir / genome_id
     else:
         file = target_dir / f'{genome_id}.{program_name}'
         if file.exists() and file.is_dir():
@@ -349,29 +359,29 @@ def write_metaerg_progress(genome_name, new_progress):
 def register_annotator(define_annotator):
     param = define_annotator()
 
-    def annotator(genome_name, contig_dict, db_connection) -> int:
+    def annotator(genome, contig_dict, db_connection) -> int:
         """Runs programs and reads results."""
         # (1) Read metaerg progress file, update progress and log the start of the analysis
         if not ANNOTATOR_STATUS[param['annotator_key']]:
             return 0
-        current_progress = parse_metaerg_progress(genome_name)
+        current_progress = parse_metaerg_progress(genome.name)
         if ANNOTATOR_STATUS[param['annotator_key']] == FORCE_ANNOTATOR or \
            ANNOTATOR_STATUS[param['annotator_key']] == UPDATE_ANNOTATOR or \
            not param['annotator_key'] in current_progress.keys():
             current_progress[param['annotator_key']] = PROGRESS_STARTED
-            write_metaerg_progress(genome_name, current_progress)
+            write_metaerg_progress(genome.name, current_progress)
 
-        log('({}) Starting {} ...', (genome_name, param['purpose']))
+        log('({}) Starting {} ...', (genome.name, param['purpose']))
 
         # (2) Make sure required databases are available
         for d in param.get('databases', []):
             d = DATABASE_DIR / d
             if not d.exists() or not d.stat().st_size:
-                log('({}) Unable to run {}, or parse results, database "{}" missing', (genome_name,
+                log('({}) Unable to run {}, or parse results, database "{}" missing', (genome.name,
                                                                                        param['purpose'], d))
                 return 0
         # (3) Then, if force or the results files are not yet there, run the programs:
-        result_files = [spawn_file(f, genome_name) for f in param.get('result_files', [])]
+        result_files = [spawn_file(f, genome.name) for f in param.get('result_files', [])]
         analysis_already_completed = True
         for f in result_files:
             if not f.exists() or not f.stat().st_size:
@@ -394,31 +404,31 @@ def register_annotator(define_annotator):
                     all_programs_in_path = False
             if all_programs_in_path:
                 try:
-                    param['run'](genome_name, contig_dict, db_connection, result_files)
+                    param['run'](genome, contig_dict, db_connection, result_files)
                 except Exception as e:
-                    log('({}) Error while running {}: {}', (genome_name, param['purpose'],
+                    log('({}) Error while running {}: {}', (genome.name, param['purpose'],
                                                             "".join(traceback.format_exception(e))))
                     return 0
             else:
-                log('({}) Unable to run {}, helper program "{}" not in path', (genome_name, param['purpose'], p))
+                log('({}) Unable to run {}, helper program "{}" not in path', (genome.name, param['purpose'], p))
                 return 0
         elif len(param.get('programs', [])):  # before logging this, do check if any result files will be parsed
-            log('({}) Reusing existing results in {}.'. format(genome_name,
+            log('({}) Reusing existing results in {}.'. format(genome.name,
                                                               ', '.join(str(file) for file in result_files)))
         # (6) Check if all results files are there, otherwise return:
         results_complete = True
         for f in result_files:
             if not f.exists() or not f.stat().st_size:
                 log('({}) Missing expected result file {}; this could mean no results were found or that a '
-                    'helper program failed to run.', (genome_name, f))
+                    'helper program failed to run.', (genome.name, f))
                 results_complete = False
         # (7) Report success, update progress
         current_progress[param['annotator_key']] = PROGRESS_COMPLETE
-        write_metaerg_progress(genome_name, current_progress)
+        write_metaerg_progress(genome.name, current_progress)
         positive_count = 0
         if results_complete:
-            positive_count = param['read'](genome_name, contig_dict, db_connection, result_files)
-        log('({}) {} complete. Found {}.', (genome_name, param['purpose'], positive_count))
+            positive_count = param['read'](genome, contig_dict, db_connection, result_files)
+        log('({}) {} complete. Found {}.', (genome.name, param['purpose'].capitalize(), positive_count))
         return 0
 
     registry.ANNOTATOR_REGISTRY[param['pipeline_position']] = annotator
