@@ -1,2 +1,2 @@
 __all__ = 'antismash aragorn functional_genes cdd cmscan diamond_and_blastn gene_writer ltr_harvest minced prodigal ' \
-          'repeat_masker signalp tmhmm trf'.split()
+          'repeat_masker signalp tmhmm trf checkm'.split()
